#!/usr/bin/perl
########################################################################
#
# $Id: sys_update.pl,v 1.15 2013-03-15 19:30:15 gosha Exp $
#
#		2012 (c) Okunev Igor <igor[at]prv.mts-nn.ru>
#
########################################################################
package VirtualPBX;

open STDERR, '>>/var/log/VirtualPBX/XVB.stderr';

use strict;

use vars qw( $VERSION );

($VERSION='$Revision: 1.15 $')=~s/^\S+\s+\d+\.(\d+)\s+.*/$1/;

########################################################################
my %update_data = (
	# top release :)
	"1000000"	=>	[
			sub { print OUT "SYS-VERSION=$VERSION, DATE=". scalar(localtime(time)) ."\n" }
	],
	
	"$VERSION"	=>	[
		sub {
			# sipusers
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-150313.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-150313.sh');
			}
		}
	],
	
	"14"	=>	[
		sub {
			# peerip => recvip
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-221012.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-221012.sh');
			}
		}
	],

	"13"	=>	[
		sub {
			# app_exec
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-061012.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-061012.sh');
			}
		}
	],

	"12"	=>	[
		sub {
			# app_while
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-060912.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-060912.sh');
			}
		}
	],

	"11"	=>	[
		sub {
			# MemCache updates
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-140812.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-140812.sh');
			}
		}
	],
	
	"9"	=>	[
		sub {
			# features tune
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-070612.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-070612.sh');
			}
		}
	],

	"8"	=>	[
		sub {
			my @files = `find '/opt/VirtualPBX/sounds/' -name 'schedule-intro.*'`;
			chomp @files;
			foreach my $file ( @files ) {
				my $new_file = $file;
				$new_file =~ s{schedule-intro}{dtmftx-intro};
				unless ( -e $new_file ) { 
					system "cp $file $new_file";
				}
			}
		}
	],

	"7"	=>	[
		sub {
			# manager tune
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-200312.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-200312.sh');
			}
		}
	],

	"6"	=>	[
		sub {
			# fix asterisk binaries
			if ( -f '/usr/lib/asterisk/modules/app_chanspy.so' && -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/app_chanspy-1.6.0.28-XVB.6.so.gz' ) {
				my $cmd = 'rpm -qv virtual-pbx';
				my $str = `$cmd`;
				if ( $str =~ /1\.6\.0\.28-XVB\.(\d+)/ and $1 < 6 ) {
					$cmd = "zcat /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/app_chanspy-1.6.0.28-XVB.6.so.gz | md5sum | awk '{print ".'$1'."}'"; 
					my $md5_1 = `$cmd`;
					$cmd = "md5sum /usr/lib/asterisk/modules/app_chanspy.so | awk '{print ".'$1'."}'"; 
					my $md5_2 = `$cmd`;
					chomp $md5_2;
					chomp $md5_1;
					if ( $md5_1 ne $md5_2 ) {
						run_sys("asterisk -rx 'module unload app_chanspy.so'");
						run_sys('zcat /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/app_chanspy-1.6.0.28-XVB.6.so.gz > /usr/lib/asterisk/modules/app_chanspy.so');
						run_sys('chmod 755 /usr/lib/asterisk/modules/app_chanspy.so');
						run_sys("asterisk -rx 'module load app_chanspy.so'");
					}
				}	
			}
		}
	],

	"5"	=>	[
		sub {
			# DID routing
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-240212.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-240212.sh');
			}
		}
	],

	"4"	=>	[
		sub {
			# ast conf
			if ( -f '/opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-210212.sh' ) {
				run_sys('bash /opt/VirtualPBX/contrib/utils/rpm/sys_update-data/upd-210212.sh');
			}
		}
	]
);

########################################################################
							main();
########################################################################

########################################################################
#
sub main {
	my $cur_version = '0';

	if ( -e '/opt/VirtualPBX/etc/.sys_version' ) {
		open( IN, '</opt/VirtualPBX/etc/.sys_version' ) || die "Can't open config file for read: $!";
		while ( <IN> ) {
			$cur_version = $1 if /SYS-VERSION=(\d+)/;
		}
		close(IN);
	}

	my $start_time = localtime;

	print "Start update system ( from $cur_version to $VERSION ): $start_time...\n";

	if ( $cur_version ne $VERSION ) {
		open( OUT, '>>/opt/VirtualPBX/etc/.sys_version' ) || die "Can't open config file for write: $!";
		foreach my $key ( sort { $a <=> $b } keys %update_data ) {
			next if $key <= $cur_version;
			foreach my $conf ( @{ $update_data{$key} } ) {
				&$conf() if ref $conf eq 'CODE';
			}
		}
		close OUT;
	}

	my $stop_time = localtime;
	print "Complite update system: $stop_time...\n";
}

########################################################################
#
sub run_sys {
	my $cmd = shift;

	if ( system($cmd) ) {
		print "RUN CMD ERROR[$cmd]: $!\n";
		die "RUN CMD ERROR[$cmd]: $!\n";
	}
}
